/*game.c
 *
 * gcc -o game -I/usr/openwin/includE -lX11 game.c
 * ./game
 */
 
#include <stdio.h>
/*
 * Header files for X functions
 */
#include <X11/Xlib.h>
#include <X11/Xutil.h>


/*
 * The different types of items used for the game.
 */
#define Air -1
#define String 0
#define Rectangle 1
#define Gninnie 2
#define Ground 3
#define Door 4
#define Exit 5
#define Obstacle 6
#define AirTangle 7
#define State 8

/*
 * The different directions a gninnie can currently be going.
 */
#define Right 0
#define Left 1


/*
 * This structure is used to represent all items that will be painted on the 
 * screen.  I took it from hello.c
 */
typedef struct {
	int type;
    int direction;
	int x, y;
    char string[100];
    int length;
    int width, height;
} Item;
        
/*
 * Data structure for a linked list.  I took it from hello.c
 */
typedef struct list{
          Item *item;
          struct list *next;
          } List, *ListPtr;

/*
 * Data structure for the GameMatrix that is used for the logic.
 */
typedef struct {
		/*Each entry in the matrix presents a 10x10 square*/
          int matrix[64][32];
} GameMatrix;



/*
 * This function paints objects to a pixmap and then copies them from the 
 * pixmap to screen in order to prevent flashing.
 */
void repaint( ListPtr head, Display *display, Window window, GC gc,Pixmap pixmap) {
	Colormap screen_colormap; 
	XColor grey, black;
	Item *it;
	Status rc;
	int holeIndex[25][2];
    int numOfHoles = 0;
 	int counter = 0;
  	int screen_num=DefaultScreen(display);
  	/* get access to the screen's color map. */
  	screen_colormap = DefaultColormap(display, DefaultScreen(display));

	//setup our pixmap by drawing our foreground.
	XSetForeground(display, gc, WhitePixel(display, screen_num));
	XFillRectangle( display, pixmap, gc, 0,0, 640, 480 );
	//setup the pixmap's foreground and background.
	XSetForeground(display, gc, BlackPixel(display, screen_num));
  	XSetBackground( display, gc, WhitePixel(display, screen_num) ); 
  	while( head ) {
    	it = head->item; 
    	switch( it->type ) {
    	case String:
    		XDrawImageString( display, pixmap, gc, it->x, it->y, it->string, it->length );
      		break;
    	case State:
    		XDrawImageString( display, pixmap, gc, it->x, it->y, it->string, it->length );
      		break;      		
		case Rectangle:
    		XDrawRectangle( display, pixmap, gc, it->x, it->y, it->width, it->height );
    		break;
		case AirTangle:
		  	holeIndex[numOfHoles][0]=it->x;
          	holeIndex[numOfHoles][1]=it->y;
		  	numOfHoles++;  
    	  	break;
		case Obstacle:
			XFillRectangle( display, pixmap, gc, it->x-(15/2), it->y-(15/2), 15, 15);    		
			break;    		    		    		
	    case Ground:
	   	  	rc = XAllocNamedColor(display, screen_colormap, "grey", &grey, &grey);
			if (rc == 0) {
  				XSetForeground(display, gc, BlackPixel(display, screen_num));
  			}
  			else{
  				rc = XAllocNamedColor(display, screen_colormap, "grey", &grey, &grey);
  				XSetForeground(display, gc, grey.pixel);
  			}
    		XFillRectangle( display, pixmap, gc, it->x, it->y, it->width, it->height );
    		XSetForeground(display, gc, BlackPixel(display, screen_num));
	    	break;
	    case Gninnie:
	    	XSetForeground(display, gc, BlackPixel(display, screen_num));
	    	XFillArc(display, pixmap, gc, it->x-(15/2), it->y-(15/2), 15, 15, 0, 360*64);
	    	break;
	    case Door:
	    	XDrawArc(display, pixmap, gc, it->x-(30/2)+15, it->y-(30/2)-30, 30, 30, 0, 180*64);
	    	XDrawLine(display, pixmap, gc, it->x, it->y, it->x,it->y-30);
	    	XDrawLine(display, pixmap, gc, it->x + 30, it->y, it->x + 30,it->y-30);
	    	break;
	    case Exit:
	    	XDrawRectangle( display, pixmap, gc, it->x, it->y-50, 60,10);
			XDrawLine(display, pixmap, gc, it->x+10, it->y, it->x+10,it->y-40);	    	
	    	XDrawLine(display, pixmap, gc, it->x+50, it->y, it->x+50,it->y-40);
	    	break;	    	
    	default:
      		printf( "repaint: Unimplemented item.\n" );
      		exit(0);
      		break;
    	}
		head = head->next;
  }
  	//paint the holes
  if (numOfHoles > 0){ 
  	for (counter=0;counter < numOfHoles; counter++){
		XSetForeground(display, gc, WhitePixel(display, screen_num));
	    XFillRectangle( display, pixmap, gc,holeIndex[counter][0] , holeIndex[counter][1], 20, 20 );
    	XSetForeground(display, gc, BlackPixel(display, screen_num));
	  }
  }
  XCopyArea(display, pixmap, window, gc,0, 0,640,480,0,0);  	
  XFlush( display );
}

/*
 * Start the program at this point.
 * A lot of this was taking from hello.c
 */
int main ( int argc, char *argv[] ) {
	ListPtr head;
	char command;
  	Display *display;
  	GC gc;
  	KeySym key;
  	XEvent event;
  	Window window;
  
  	int i;
  	char text[10];
	
	unsigned long background, foreground;
  	int screen;
	XSizeHints hints;
  	
   /*
 	* Display opening uses the DISPLAY  environment variable.
 	* It can go wrong if DISPLAY isn't set, or you don't have permission.
 	*/  
  	display = XOpenDisplay( "" );
  	if ( !display ) {
    	printf( "Can't open display." );
    	exit(1);
  	}
  	screen = DefaultScreen( display );
	background = WhitePixel( display, screen ); 
  	foreground = BlackPixel( display, screen );
	/*
 	* Set up hints and properties for the window manager, and open a window.
 	*/
  	hints.x = 100; 
  	hints.y = 100; 
  	hints.width = 640; 
  	hints.height = 480; 
  	hints.flags = PPosition | PSize;
  	window = XCreateSimpleWindow( display, DefaultRootWindow( display ), hints.x, hints.y,
                                hints.width, hints.height, 5, foreground, background );
  	XSetStandardProperties( display, window, "Gnimmel: The Gname", "Gnimmel: The Gname", None, argv, argc, &hints );
	/*
 	* Get a graphics context and set the drawing colours.
 	*/
  	gc = XCreateGC (display, window, 0, 0 );
  	XSetBackground( display, gc, background ); 
  	XSetForeground( display, gc, foreground );
	/*
 	* Tell the window manager what input you want.
 	*/
  	XSelectInput( display, window, ButtonPressMask | KeyPressMask | ExposureMask );
	/*
 	* Put the window on the screen.
 	*/
  	XMapRaised( display, window );
	head = 0;
  	startInitialSplashScreen(display,window,gc,event);
  	command = '1';
  	while (1){
  		switch (command){
  			case '1':
  				command = setupLevelOne(display,window,gc,event);
  				break;
  			case '2':
  				//command = setupLevelTwo(display,window,gc,event);
  				break;
  			case '3':
  				//command = setupLevelThree(display,window,gc,event);
  				break;
  			case 't':
  				//command = toggleImplementations;
  				break;
  		}
  	}

}

void startInitialSplashScreen(Display * display,Window window,GC gc,XEvent event){	
	ListPtr head = 0;
	int i;
	KeySym key;
	char text[10];
	head = insertItem(head, mkRectangle(232,15,176,30,Rectangle));
	head = insertItem(head, mkRectangle(10,175,250,20,Ground));
	head = insertItem(head, mkRectangle(260,235,170,20,Ground));
	head = insertItem(head, mkRectangle(430,295,150,20,Ground));	
	head = insertItem(head, mkStringItem( 234, 40, "Welcome to Gnimmel: The Gname"));
	head = insertItem(head, mkStringItem(1,345, "The object of Gnimmel : The Gname is to safely lead the gninnies from the start point to the exit."));
	head = insertItem(head, mkStringItem(1,360, "Gninnies (represented as a full circle) can be made to perform the following actions;"));
	head = insertItem(head, mkStringItem(1,375, "1)Turn into an obstacle, forcing other gninnies to bounce off it, with the centre mouse button."));
	head = insertItem(head, mkStringItem(1,390, "2)Cease being an obstacle, resuming it's previous behavior, with the centre mouse button."));	
	head = insertItem(head, mkStringItem(1,405, "3)Dig through the ground, allowing gninnies to follow, with the right mouse button."));	
	head = insertItem(head, mkStringItem(1,420, "To quit the game at any time after this screen, press the Q key on the keyboard."));	
	head = insertItem(head, mkStringItem(1,465, "Press the C key on the keyboard to continue."));	
	head = insertItem(head, mkGninnie(25,10));

	/* this pixmap code was taken from the tutorial in the assignment
	 * resources site.
	 */
	Pixmap pixmap;
	Window root_win = DefaultRootWindow(display);
    int depth = DefaultDepth(display, DefaultScreen(display));
    pixmap = XCreatePixmap(display, root_win, 640, 480, depth);

	repaint( head, display, window, gc,pixmap);
	
  	while( 1 ) {
		if (XEventsQueued(display,QueuedAfterReading) != 0){
    		XNextEvent( display, &event );
		    switch( event.type ) {
				/*
				 * Repaint the window on expose events.
				 */
			    case Expose:
		    		if ( event.xexpose.count == 0 ) {
				    	repaint( head, display, window, gc,pixmap);
				    }
					break;
				/*
			 	* Exit when 'q' is typed.
			 	*/
		    	case KeyPress:
		    		i = XLookupString( (XKeyEvent *)&event, text, 10, &key, 0 );
    				if ( i == 1 && text[0] == 'c' ) {
						removeAllItems(head);
						XFreePixmap(display, pixmap);
					  	XClearWindow( display, window );    			
				    	return;  
      				}
      				break;
      		}
		}
      	animateSplashScreen(head);
      	repaint( head, display, window, gc,pixmap); 
      	usleep( 3000  );   	
	} 	
}

/*
 * This method does a dummy animatation for the splash screen.
 */
void animateSplashScreen(ListPtr head){
	Item * gninnie = head->item;
	if (gninnie->y < 165){
		gninnie->y = gninnie->y +10;
		return;
	}
	if (gninnie->x < 268){
		gninnie->x = gninnie->x +10;		
		return;
	}
	if (gninnie->y < 229){
		gninnie->y = gninnie->y +10;
		return;		
	}
	if (gninnie->x < 438){
		gninnie->x = gninnie->x +10;		
		return;
	}
	if (gninnie->y < 288){
		gninnie->y = gninnie->y +10;
		return;		
	}
	if (gninnie->x < 588){
		gninnie->x = gninnie->x +10;		
		return;
	}
	if (gninnie->y < 480){
		gninnie->y = gninnie->y +10;
		return;		
	}
	head = removeItem(head,gninnie);
	head = insertItem(head, mkGninnie(25,10));	
}
   
/*
 * Insert an item at the head of the list.
 * This item will be drawn first so it will be at the back of the image.
 * Taken from hello.c
 */
ListPtr insertItem ( ListPtr head, Item *itp ) {
  ListPtr new;  
  new = ( ListPtr ) malloc( sizeof( List ) );
  new->item = itp; new->next = head;
  return new;
}

/*
 * Function to make a new String item; a rectangle item is similar.
 * Taken from hello.c
 */
Item *mkStringItem( int x, int y, char *str ) {
  Item *new;
  new = ( Item * ) malloc ( sizeof( Item ) );
  new->type = String; new->x = x; new->y = y; strcpy( new->string, str ); new->length = strlen( str );
  return new;
}

/*
 * Function to make a new String item; a rectangle item is similar.
 * Taken from hello.c
 */
Item *mkStatusItem( int total, int target,int entered,int exited,int died) {
  char str[200];
  Item *new;
  sprintf(str, "Level Status - Gninnies Entered:%i Gninnies to Arrive:%i Dead Gninnies:%i Gninnies Exited:%i Target:%i", entered,total-entered,died,exited,target) ;
  new = ( Item * ) malloc ( sizeof( Item ) );
  new->type = State; new->x = 3; new->y = 400; strcpy( new->string, str ); new->length = strlen( str );
  return new;
}

/*
 * Function to make a rectangle item.
 * Modified from hello.c
 */
Item *mkRectangle( int x, int y, int width, int height,int rectType ) {
  Item *new;
  new = ( Item * ) malloc ( sizeof( Item ) );
  new->type = rectType; 
  new->x = x; 
  new->y = y; 
  new->width = width; 
  new->height = height;
  return new;
}

/*
 * Make a Gninnies.
 */
Item *mkGninnie( int x, int y) {
  Item *new;
  new = ( Item * ) malloc ( sizeof( Item ) );
  new->type = Gninnie; 
  new->x = x; 
  new->y = y;
  new->direction = Right;
  return new;
}

/*
 * Function to make terrain object.
 */
Item *mkTerrainObject( int x, int y,int type) {
  Item *new;
  new = ( Item * ) malloc ( sizeof( Item ) );
  new->type = type; 
  new->x = x; 
  new->y = y;
  return new;
}

/*
 * Removes an item from the linked list.
 * Taken from hello.c
 */
ListPtr removeItem( ListPtr head, Item *itp ) {
  ListPtr current, previous, tmp;
  if ( !head ) {
    return head;
  } 
  if ( head->item == itp ) {
    tmp = head; head = head->next;
    free( tmp->item ); free( tmp );
    return head;
  }
  current = head->next; previous = head;
  while ( current ) {
    if ( current->item == itp ) {
      previous->next = current->next; tmp = current; current = current->next;
      free( tmp->item ); free( tmp );
    } else {
      previous = current; current = current->next;
    }
  }
  return head;
}

/*
 * Removes all items from a linked list.
 */
void removeAllItems(ListPtr head){
	ListPtr temp;
	temp = head;
	while( head ) {
		temp = head->next; 
		free( head->item ); 
		free( head );
		head = temp;
    }
    return;
}

/*
 * setups the first level.
 * returns a char depending on what command was sent.
 */
char setupLevelOne(Display * display,Window window,GC gc,XEvent event){
	ListPtr head = 0;
	int i;
	char command;
	int x;
	int y;	
	KeySym key;
	char text[10];
	head = insertItem(head, mkRectangle(300,15,46,30,Rectangle));
	head = insertItem(head, mkRectangle(0,150,320,20,Ground));
	head = insertItem(head, mkRectangle(0,300,600,20,Ground));
	head = insertItem(head, mkTerrainObject(40,150,Door));
	head = insertItem(head, mkTerrainObject(540,300,Exit));	
	head = insertItem(head, mkStringItem( 302, 40, "Level 1"));
	head = insertItem(head, mkStringItem( 3, 85, "Click on the Entrance with the left mouse button to begin the level."));	

	/* this pixmap code was taken from the tutorial in the assignment
	 * resources site.
	 */
	Pixmap pixmap;
	Window root_win = DefaultRootWindow(display);
	int depth = DefaultDepth(display, DefaultScreen(display));
	pixmap = XCreatePixmap(display, root_win, 640, 480, depth);
	
	repaint( head, display, window, gc,pixmap);
  	while( 1 ) {
  		if (XEventsQueued(display,QueuedAfterReading) != 0){
    		XNextEvent( display, &event );
		    switch( event.type ) {
				/*
				 * Repaint the window on expose events.
				 */
			    case Expose:
		    		if ( event.xexpose.count == 0 ) {
				    	repaint( head, display, window, gc,pixmap);
				    }
					break;
 				case ButtonPress:

			    	/* store the mouse button coordinates in 'int' variables. */
				    /* also store the ID of the window on which the mouse was */
				    /* pressed.                                               */
				    x = event.xbutton.x;
				    y = event.xbutton.y;
				    
				    if (event.xbutton.button == Button1
				    && (x>=40 && x<=70) && (y<=150 && y>=95)){
				    	head = removeItem(head,head->item);
				    	repaint( head, display, window, gc,pixmap);
				    	command = startLevelOne(head, display, window, gc,event,pixmap);
				    	if (command == 'n'){
 						  	XClearWindow( display, window );
 						  	removeAllItems(head);
 						  	insertItem(head,mkStringItem(291,235,"You have failed level 1.  The level will now be restarted."));
 						  	repaint(head,display,window,gc,pixmap);
				  	  		usleep(7000000);
							XFreePixmap(display, pixmap);
						  	XClearWindow( display, window );
				    		return '1';
				    	}
				    	else if (command == 'y'){
 						  	XClearWindow( display, window );
 						  	removeAllItems(head);
 						  	insertItem(head,mkStringItem(296,235,"You have passed level 1.  Level 2 will now begin."));
 						  	repaint(head,display,window,gc,pixmap);                         
				  	  		usleep(7000000);
				    		XFreePixmap(display, pixmap);
					  		XClearWindow( display, window );				    		
				    		return '2';
				    	}
				    	else{
				    		XFreePixmap(display, pixmap);
 						  	XClearWindow( display, window );
				    		return command;
				    	}
				    	
				    }
				    
				    break;
				
				/*
			 	* React to the different hotkeys in the game.
			 	*/
		    	case KeyPress:
		    		i = XLookupString( (XKeyEvent *)&event, text, 10, &key, 0 );
    				if ( i == 1 && text[0] == 'q' ) {
						exit(0);
      				}
    				if ( i == 1 && text[0] == '1' ) {
    					XFreePixmap(display, pixmap);
					  	XClearWindow( display, window );    					
						return '1';
      				}
    				if ( i == 1 && text[0] == '2' ) {
    					XFreePixmap(display, pixmap);
					  	XClearWindow( display, window );    					
						return '2';
      				}
    				if ( i == 1 && text[0] == '3' ) {
    					XFreePixmap(display, pixmap);
					  	XClearWindow( display, window );    					
						return '3';
      				}
    				if ( i == 1 && text[0] == 't' ) {
    					XFreePixmap(display, pixmap);
					  	XClearWindow( display, window );    					
						return 't';
      				}    				      	      				      				       				      				
      				break;
      		}
		}
	} 
		
}

/*
 * This method actually starts the first level.
 */
char startLevelOne(ListPtr head,Display * display,Window window,GC gc,XEvent event,Pixmap pixmap){
	int i;
	KeySym key;
	char text[10];	
	int x,y;
	int counter;
	int entered,exited,target,died,total;
	char command;
	int paused;
	GameMatrix *gameMatrix;
  	gameMatrix = ( GameMatrix * ) malloc ( sizeof( GameMatrix ) );
  	
  	paused=0;
  	entered=0;
  	exited=0;
  	total=2;
  	target=2;
  	died=0;
	//set everything to air.
	for (x = 0;x<64;x++){
		for (y = 0; y < 32;y++){
			gameMatrix->matrix[x][y]= Air;
		}
	}
	
	//setup the ground.
	for (x = 0; x < 32;x++){
		for (y = 15;y < 17;y++){
			gameMatrix->matrix[x][y]= Ground;
		} 
	}

	//setup the exit.
	for (x = 0;x < 4;x++){
		for (y = 24;y < 30;y++){
			gameMatrix->matrix[x+55][y]=Exit;
		} 
	}

	//setup the ground.
	for (x = 0; x < 60;x++){
		for (y = 30;y < 32;y++){
			gameMatrix->matrix[x][y]=Ground;
		} 
	}
		
	head = insertItem(head, mkGninnie(50,140));
	gameMatrix->matrix[5][14];
	entered++;
	
	//initailize our exit and dead values.
	gameMatrix->matrix[0][0] = 0;
	gameMatrix->matrix[0][1] = 0;
	
	//count used to determine when to release next gninnie.
	counter = 0;
	head = insertItem(head,mkStatusItem(total, target,entered,exited,died));
	repaint( head, display, window, gc,pixmap);	
  	while( 1 ) {
		
  		usleep(7500);
  		counter++;
  		if (counter == 100){
			head = insertItem(head, mkGninnie(50,140));
			gameMatrix->matrix[5][14];
			entered++;  			
  		}
  	       
  		if (XEventsQueued(display,QueuedAfterReading) != 0){
    		XNextEvent( display, &event );
		    switch( event.type ) {
				/*
				 * Repaint the window on expose events.
				 */
			    case Expose:
		    		if ( event.xexpose.count == 0 ) {
				    	repaint( head, display, window, gc,pixmap);
				    }
					break;
 				case ButtonPress:

			    	/* store the mouse button coordinates in 'int' variables. */
				    /* also store the ID of the window on which the mouse was */
				    /* pressed.                                               */
				    x = event.xbutton.x;
				    y = event.xbutton.y;
				    ListPtr clickedGninnie = checkIfGninniewasClicked(head,x,y);
				    if (clickedGninnie){
    					/* check which mouse button was pressed, and act accordingly. */
				      switch (event.xbutton.button) {
        					case Button1:
            					break;
        					case Button2:
            					/* If the center button is clicked */
            					toggleGninnieObstacle(clickedGninnie,gameMatrix);
            					break;
        					default:
						  if (clickedGninnie->item->type == Gninnie){
        						//right mouse button is clicked.
        					head = haveGninnieDig(head,clickedGninnie,gameMatrix);
						  }
            					break;
    					}
				    }
				    break;
				
				/*
			 	* React to the different hotkeys in the game.
			 	*/
		    	case KeyPress:
		    		i = XLookupString( (XKeyEvent *)&event, text, 10, &key, 0 );
    				if ( i == 1 && text[0] == 'q' ) {
						exit(0);
      				}
    				if ( i == 1 && text[0] == '1' ) {

					  	XClearWindow( display, window );    					
						return '1';
      				}
    				if ( i == 1 && text[0] == '2' ) {

					  	XClearWindow( display, window );    					
						return '2';
      				}
    				if ( i == 1 && text[0] == '3' ) {

					  	XClearWindow( display, window );    					
						return '3';
      				}
    				if ( i == 1 && text[0] == 't' ) {

					  	XClearWindow( display, window );    					
						return 't';
      				}
    				if ( i == 1 && text[0] == 'p' ) {
   						head = insertItem(head, mkStringItem( 3, 85, "The game is paused.  Press R to resume the game or Q to quit."));	
					  	repaint( head, display, window, gc,pixmap);
					  	/*enter a mini event loop for the pause */
					  	paused = 1;
						while (paused){
							XNextEvent( display, &event );
		    				switch( event.type ) {
							/*
							 * Repaint the window on expose events.
							 */
			    			case Expose:
		    					if ( event.xexpose.count == 0 ) {
							    	repaint( head, display, window, gc,pixmap);
							    }
								break;
		    				case KeyPress:
		    					i = XLookupString( (XKeyEvent *)&event, text, 10, &key, 0 );
    							if ( i == 1 && text[0] == 'q' ) {
									exit(0);
      							}
      							else if (i == 1 && text[0] == 'r'){
      								paused = 0;
									head = removeItem(head,head->item);
									repaint( head, display, window, gc,pixmap);      								
      								break;
      							}
      							break; 							
		    				}
						}

      				}      				    				      	      				      				       				      				
      				break;
      		}
		}
      	head = animateGame(head,gameMatrix);
      	exited = gameMatrix->matrix[0][0];
      	died = gameMatrix->matrix[0][1];
      	if (exited >= target){
      		return 'y';
      	}
      	if (total - target < died){
      		return 'n';
      	}
      	head = removeItem(head,returnStatusItem(head));
      	head = insertItem(head,mkStatusItem(total, target,entered,exited,died));
      	repaint( head, display, window, gc,pixmap);
	}
 	
}

/*
 * This method performs the animation for the game.
 */
ListPtr animateGame (ListPtr head, GameMatrix * gameMatrix){

	Item * item;
	int i;
	ListPtr temp;
	temp = head;
	while( temp ) {
		item=temp->item;
		if (item->type == Gninnie){
			i = animateGninnie(item,gameMatrix);
			if (i == 1){
				/*if a gninnie has exited, keep track of this
				in a special cell and remove the Gninnie from the game*/
				head = removeItem(head,item);
				gameMatrix->matrix[0][0] = gameMatrix->matrix[0][0] +1;
		 	}
		 	else if (i == 2){
				/*if a gninnie has died remove them from the game
				 * and store the fact that they died*/		 		
				head = removeItem(head,item);
				gameMatrix->matrix[0][1] = gameMatrix->matrix[0][1] +1;
		 	}
		}
		temp = temp->next;
    }

    return head;	
}

/*
 * This method actually animates the gninnies.
 */
int animateGninnie(Item * item,GameMatrix * gameMatrix){
	int yMatrix,xMatrix;

	if (item->type == Obstacle){
		return 0;
	}
	/*otherwise our Gninnie is walking.
	*See if the Gninnie is above air.
	*/
	xMatrix = item->x / 10;
	yMatrix = item->y / 10;

	if (gameMatrix->matrix[xMatrix][yMatrix+1] == Air){
		/*if the spot under the gninnie is air, he should fall.*/
	       
		item->y = item->y + 1;
		if (yMatrix == 31){
			//if they have fallen off the screen.
			return 2;
		}
		return 0;
	}
	
	/*	This means the Gninnie is standing on ground, an obstacle, or the exit*/
	if (item->direction == Right){
	  if(gameMatrix->matrix[xMatrix+1][yMatrix] == Obstacle){
	    item->direction = Left;
	    return 0;
	  }
		item->x = item->x + 1;
		if (item->x > 640){
			/*this gninnie is dead for going out of frame*/
		  return 2;
		}
	}
	else {
	  if(gameMatrix->matrix[xMatrix-1][yMatrix] == Obstacle){
	    item->direction = Right;
	    return 0;
	  } 
	  /*The gninnie is going left*/
	  item->x = item->x - 1;
	  	if (item->x < 0){
			/*this gninnie is dead for going out of frame*/
		  	return 2;
		}
	
	}
	xMatrix = item->x /10;
	if (gameMatrix->matrix[xMatrix][yMatrix] == Exit){
		return 1;
	}
	else {
		return 0;
	}	
}

/*
 * This method determines if a gninnie was clicked.
 */
ListPtr checkIfGninniewasClicked(ListPtr head, int x,int y){
	Item * it;
	while(head){
		it = head->item;
		if (it->type == Gninnie || it->type == Obstacle){
			if ((x >= it->x -15 && x <= it->x + 15) &&
				(y >= it->y -15 && y <= it->y + 15)){
					return head;
				}
		}
		head = head->next;
	}
	return NULL;
}

/*
 * This method turns a gninnie into an obstacle and vice versa.
 */
void toggleGninnieObstacle(ListPtr pointer,GameMatrix * gameMatrix){
	Item * item;
	item = pointer->item;
	int xMatrix = item->x / 10;
	int yMatrix = item->y / 10;
	if (item->type == Obstacle){
	  	gameMatrix->matrix[xMatrix][yMatrix] = Air;
		item->type = Gninnie;
	}
	else if (item->type == Gninnie && gameMatrix->matrix[xMatrix][yMatrix+1]!=Air){
	  	gameMatrix->matrix[xMatrix][yMatrix] = Obstacle;
		item->type = Obstacle;
	}
	return;
}

/* if the gninnie is standing on ground have them dig.*/
ListPtr haveGninnieDig(ListPtr head,ListPtr pointer,GameMatrix * gameMatrix){
	Item * item;
	item = pointer->item;
	int xMatrix = item->x / 10;
	int yMatrix = item->y / 10;

	if (gameMatrix->matrix[xMatrix][yMatrix+1] == Ground){
		gameMatrix->matrix[xMatrix][yMatrix+1] = Air;
		gameMatrix->matrix[xMatrix][yMatrix+2] = Air;
	    return insertItem(head,mkRectangle(item->x-8,item->y+10,20,20,AirTangle));
	}
	else {
	  	return head;
	}
}

Item * returnStatusItem(ListPtr head){
	Item * it;
	while(head){
		it = head->item;
		if (it->type == State){
			return it;
		}
		head = head->next;
	}
	return NULL;	
}


